// fichier : date.cc
// version : 1.0
// 
#include <iostream>
#include <limits>
#include <vector>

using namespace std;
#include "date.h"


static constexpr unsigned starting_year(1600);// zero ref for current calendar

static bool date_correct(unsigned dd, unsigned mm, unsigned yy);
static unsigned yy_is_leap_year(unsigned yy);
static unsigned day_per_month (unsigned mm, unsigned yy);
static unsigned day_per_year(unsigned yy);
static unsigned nb_of_days_since_starting_year(unsigned dd, unsigned mm, unsigned yy);

Date::Date(unsigned dd, unsigned mm, unsigned yy)
{
	if(date_correct(dd,mm,yy))
	{
		day   = dd;
		month = mm;
		year  = yy;
	}
	else
	{
		cout << "Date impossible dans le calendrier actuel\n" << endl;
		cout << "La Date est initialisée au 1.1.1600 \n" << endl;
		day   = 1;
		month = 1;
		year  = 1600;
	}
}

bool Date::operator==(const Date& b)
{
	return(day    == b.day    && 
	       month  == b.month  &&
	       year   == b.year );
}

bool Date::operator!=(const Date& b)
{
	return( not ((*this) == b));
}

// the special character '\t' is a tabulation
void	 Date::affiche() const
{
	cout << day << " \t" << month << " \t" << year << endl; 
}


unsigned Date::nb_of_days_between_dates(const Date& d) const
{
  return abs(int(nb_of_days_since_starting_year(day    ,month,  year)) -
	         int(nb_of_days_since_starting_year(d.day,d.month,d.year)));
}


static bool date_correct(unsigned dd, unsigned mm, unsigned yy)
{ 
  if ((mm < 1) || (mm > 12))
    return false;

  if ((dd < 1) || (dd > day_per_month (mm, yy)))
    return false;
    
  if(yy < starting_year)
    return false;
  
  return true;   
}

static unsigned yy_is_leap_year(unsigned yy)
{
	if((yy % 400) == 0)
		return true;
	else if((yy % 100) == 0)
		return false;
	else
		return ((yy % 4) == 0);
}

static unsigned day_per_month (unsigned mm, unsigned yy)
{ 
	switch (mm)
    {
    case 4 :
    case 6 :
    case 9 :
    case 11: 
		return 30;      
    case 1 :
    case 3 :
    case 5 :
    case 7 :
    case 8 :
    case 10:
    case 12: 
		return 31;      
    case 2 : 
		return yy_is_leap_year (yy) ? 29 : 28;      
    default: 
		cout << "Erreur: je ne connais pas le mois "
		     << mm << endl;
		return 0;
    }
}

static unsigned day_per_year(unsigned yy)
{ 
  return yy_is_leap_year (yy) ? 366 : 365;
}

static unsigned nb_of_days_since_starting_year(unsigned dd, unsigned mm, unsigned yy)
{
	unsigned total(0);

	for (unsigned i(starting_year); i < yy; i++)
		total += day_per_year (i);
  
	for (unsigned i(1); i < mm; i++)
		total += day_per_month (i, yy);
  
	return total + dd - 1;
}
