#include <algorithm>
#include <cmath>
#include <iostream>
#include <sstream>
#include <string>

#include "form.h"
using namespace std;

// Constructor for squares and circles, and constructor by default (from slides): 
// p1 acts as the side length of the square 
// or the radius of the circle
Form::Form(Category c, double x, double y, double p1)  
    :category(c), x(x), y(y)
{
    switch(category)
    {
        case CIRCLE:
        case SQUARE: 
            if(p1 < 0.)
            {
                cout << "Form: incorrect parameter value" << endl;
                exit(EXIT_FAILURE);
            }
            param[0] = p1;
            break;
        default: 
            cout << "Form: incorrect category" << endl;
            exit(EXIT_FAILURE);
    }
}

// Constructor for rectangles (from slides):
// p1 and p2 act as length and width of the rectangle
Form::Form(Category c, double x, double y, double p1, double p2)  
	:category(c), x(x), y(y)
{
    if(category == RECTANGLE)
    { 
        if(p1<0. || p2<0)
        {
            cout << "Form: incorrect parameter value" << endl;
            exit(EXIT_FAILURE);
        }
        param[0] = p1;
        param.push_back(p2);
     }
     else
     {
        cout << "Form: incorrect category" << endl;
        exit(EXIT_FAILURE);
     }
}

// display the basic attributes (from the slides)
void Form::affiche()  
{
    switch(category)
    {
        case CIRCLE: 
            cout << "CIRCLE with center:(" << x << ", " << y 
                 << " ) and radius " << param[0] << endl;
            break;
        case SQUARE: 
            cout << "SQUARE with center:(" << x << ", " << y 
                 << " ) and side " << param[0] << endl;
            break;
        case RECTANGLE: 
            cout << "RECTANGLE with center:(" << x << ", " << y 
                 << " ), width: " << param[0] << " and height: " << param[1] << endl;
            break;
        default: 
            cout << "Form: incorrect category" << endl;
            exit(EXIT_FAILURE);
    }
} 


Category Form::get_category() 
{
    return category;
}

vector<double> Form::get_param() 
{
    return param;
}

double Form::get_x() 
{
    return x;
}

double Form::get_y() 
{
    return y;
}

// return the smaller rectangular bounding box
Bbox Form::get_bbox() 
{
    Bbox bbox = *this;
    switch(category)
    {
        case RECTANGLE:
            break;
        case CIRCLE:
            bbox.category = RECTANGLE;
            bbox.param[0] = param[0]*2;
            bbox.param.push_back(param[0]);
            break;
        case SQUARE: 
            bbox.category = RECTANGLE;
            bbox.param.push_back(param[0]);
            break;
        default: 
            cout << "Form Bbox: incorrect category" << endl;
            exit(EXIT_FAILURE);
    }
    return bbox;
}

Bbox Form::merge_bbox(Bbox& other)
{
    double x_min, y_min, x_max, y_max;

	if(category == RECTANGLE && other.category == RECTANGLE)
    { 
		x_min = min(x - param[0]/2, other.x - other.param[0]/2);
		y_min = min(y - param[1]/2, other.y - other.param[1]/2);
		x_max = max(x + param[0]/2, other.x + other.param[0]/2);
		y_max = max(y + param[1]/2, other.y + other.param[1]/2);

		return Bbox(RECTANGLE, (x_min+x_max)/2, (y_min+y_max)/2, 
		                       (x_max-x_min),   (y_max-y_min));
	}
	else
	{
        cout << "Bbox: incorrect category" << endl;
        exit(EXIT_FAILURE);
	}
}

bool Form::intersect_bbox(Bbox& other)
{
	double x_distance, y_distance, x_border, y_border;

	if(category == RECTANGLE && other.category == RECTANGLE)
    { 
		x_distance  = fabs(x - other.x);
		y_distance  = fabs(y - other.y);

		x_border = (param[0] + other.param[0])/2;
		y_border = (param[1] + other.param[1])/2;

		if((x_distance < x_border) && (y_distance < y_border))
            return true;
 		else
            return false;
	}
	else
	{
		cout << "Bbox: incorrect category" << endl;
		exit(EXIT_FAILURE);
	}
	
}

ostream & operator<<(ostream & o, Form & f)
{
	f.affiche();
    return o;
}


Form form_lecture(istringstream& iss, Category c)
{
	double x, y, p1, p2 = 0;
	Form new_form;
    switch(c)
    {
        case CIRCLE:
            iss >> x >> y >> p1;
            new_form = Form(CIRCLE, x, y, p1);
            break;
        case SQUARE: 
            iss >> x >> y >> p1;
            new_form = Form(SQUARE, x, y, p1);
            break;
        case RECTANGLE: 
            iss >> x >> y >> p1 >> p2;
            new_form = Form(RECTANGLE, x, y, p1, p2);
            break;
    }
	return new_form;
}

