
#include <iostream>
#include <algorithm>
#include <cmath>
#include <vector>
#include <fstream>
#include <sstream>

#include "formset.h"

using namespace std;

enum Etat_lecture {NB_ELEM, ELEM_CIRCLE, ELEM_SQUARE, ELEM_RECTANGLE, FIN};

static Etat_lecture next(size_t nbc, size_t nbs, size_t nbr);

// set of FormSets maintained hidden within this module
static vector<Formset> tab;

Bbox Formset::get_bbox()
{
    return bbox_of_set;
}

void Formset::add(Form form_to_add)
{
    formset.push_back(form_to_add);
    Bbox bbox_to_add = form_to_add.get_bbox();

    if(formset.size() == 1)  // first added Form
    {
        bbox_of_set = bbox_to_add;
    }
    else  // merge current bbox of the formset with the one of the added Form
    {
		bbox_of_set = bbox_of_set.merge_bbox(bbox_to_add);
    }
}

bool Formset::interset_formset(Formset & fs)
{
	return bbox_of_set.intersect_bbox(fs.bbox_of_set);
}

void Formset::affiche()
{
    for(size_t i=0; i<formset.size(); i++)
        cout << "Form " << i+1 << " -- " << formset[i];
}

// renvoie true si aucune intersection avec le vector de Formset caché dans le module
bool Formset::no_intersect_with_set()
{
	for(size_t i=0; i < tab.size(); i++)
	{
		if(bbox_of_set.intersect_bbox(tab[i].bbox_of_set))
		{
			return false;
		}
	}
	return true;
}

ostream & operator<<(ostream & o, Formset & fs)
{
	fs.affiche();
    return o;
}

static Etat_lecture next(size_t nbc, size_t nbs, size_t nbr) // useful only below
{
	return (nbc>0)? ELEM_CIRCLE:((nbs>0)? ELEM_SQUARE:((nbr>0)? ELEM_RECTANGLE:FIN));
}

void formset_lecture(ifstream & ifs, Formset & fs)
{
    size_t nbcircle(0), nbsquare(0), nbrectangle(0);
    string line;
    Etat_lecture etat(NB_ELEM);
    Form new_form;

	while(getline(ifs>>ws, line))
	{
		if(line[0]=='#') continue;
		istringstream iss(line);
		switch(etat)
		{
			case NB_ELEM:	
				iss >> nbcircle >> nbsquare >> nbrectangle;
				etat = next(nbcircle, nbsquare, nbrectangle);
				break;
			case ELEM_CIRCLE:
				new_form = form_lecture(iss, CIRCLE);
				fs.add(new_form);
				--nbcircle;
				etat = next(nbcircle, nbsquare, nbrectangle);
				break;
			case ELEM_SQUARE:
				new_form = form_lecture(iss, SQUARE);
				fs.add(new_form);
				--nbsquare;
				etat = next(nbcircle, nbsquare, nbrectangle);
				break;
			case ELEM_RECTANGLE:
				new_form = form_lecture(iss, RECTANGLE);
				fs.add(new_form);
				--nbrectangle;
				etat = next(nbcircle, nbsquare, nbrectangle);
				break;
			case FIN: // there should be no moreline to analyze 
			default:  cout << "Error decoding Formset" << endl;
		}
		if(etat == FIN) return;
	}
}

void add_formset(Formset & fs)
{
	tab.push_back(fs);
}

void affiche_formset_valid()
{
	cout << " affichage des Formset qui ont été lus avec validation  " << endl;
	
	for(size_t i=0; i<tab.size(); i++)
	{
		cout << "Formset: "<< i+1 << endl  
		     << tab[i]     << endl;
	}
}










